
//========================================================================//
// date : 2012/06/08
// brief : part of lighting
// Created by Takashi Shibahara.
//========================================================================//
//- includes ---------------------------------------------------------------

//- define -----------------------------------------------------------------

//- constance number -------------------------------------------------------
cbuffer changeOnObj
{
	matrix wvp;
	matrix world;
	float3 eyePos;			// world coord
	float4 coloration;		// diffuse̕ϐFp
}

#define	LIGHT_MAX	5
cbuffer changeOnFrame
{
	int lightNum;
	float3 directions[LIGHT_MAX];
	float4 colors[LIGHT_MAX];
};

//- structures difinition --------------------------------------------------


//- functions --------------------------------------------------------------
//--------------------------------------------------------------------------
// brief : calculate lighted color
// in 	 : pos, normal, mesh diffuse
// out   : lighted color
// note  : pos and normal must be in local coord
//--------------------------------------------------------------------------
float4 CalculateLight( float4 pos, float3 normal, float4 color )
{
	float4 diffuse = (float4)0;
	float4 specular = (float4)0;
	

	for( int i=0; i<lightNum; ++i )
	{
		float3 eyeToPos = normalize( (float3)mul(pos, world) - eyePos );
		float NL = dot( -directions[i], normalize( mul(normal, (float3x3)world) ) );
		diffuse += saturate( NL * colors[i] );
	
		const float SPEC_COEF = 0.7;
		const float SPEC_SHINE = 10;

		/* phong
		float3 reflect = 2.0 * dot( normal, -eyeToPos ) * normal;
		float4 specular = colors[i] * SPEC_COEF * pow( saturate( dot(-directions[i], reflect ) ), SPEC_SHINE );
		//*/

		//* blinn-phong
		float3 half = normalize( -directions[i] + eyeToPos );
		specular += colors[i] * SPEC_COEF * pow( saturate( dot(normal, half ) ), SPEC_SHINE );
		//*/

	}

	diffuse.a = 1.0;

	const float AMBIENT_PARAM = 0.72;
	const float4 ambient = { AMBIENT_PARAM, AMBIENT_PARAM, AMBIENT_PARAM, 0.0 };

	float4 result = color * (diffuse + specular) + ambient;
	//float4 result = color * (diffuse) + ambient;
	return result;
}



// this is snipet for copy
//--------------------------------------------------------------------------
// brief : 
// in 	 : 
// out   : 
// note  : nothing
//--------------------------------------------------------------------------
