
//========================================================================//
// date : 2012/05/--
// brief : shader of textured obj
// Created by Takashi Shibahara.
//========================================================================//
//- includes ---------------------------------------------------------------

//- constance number -------------------------------------------------------
#include	"Lighting.fxh"

texture	diffuseTex;

sampler diffuseTexSampler = sampler_state
{
    Texture = <diffuseTex>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//- structures difinition --------------------------------------------------
struct tagVS_INPUT
{
	float4 pos : POSITION;
    float4 color : COLOR;
	float3 normal : NORMAL;
	float2 diffuseUV : TEXCOORD0;
};

#define PER_PIXEL_LIGHTING	0

#if	PER_PIXEL_LIGHTING
struct tagPS_INPUT
{
    float4 pos : SV_POSITION;
    float4 color : COLOR;
	float2 diffuseUV : TEXCOORD0;
	float3 normal : TEXCOORD1;
};

#else

struct tagPS_INPUT
{
    float4 pos : SV_POSITION;
    float4 color : COLOR;
	float2 diffuseUV : TEXCOORD0;
};

#endif

//- vertex shader ----------------------------------------------------------
//--------------------------------------------------------------------------
// brief : ordinal vertex shader
// in 	 : 
// out   : 
// note  : nothing
//--------------------------------------------------------------------------
tagPS_INPUT VS( tagVS_INPUT input )
{
	tagPS_INPUT output = (tagPS_INPUT)0;
    output.pos = mul( input.pos, wvp );

#if	PER_PIXEL_LIGHTING
	output.color = input.color;

#else
	float4 lightParam =
		CalculateLight( input.pos, input.normal, input.color );

	output.color = lightParam * coloration;
#endif

	output.diffuseUV = input.diffuseUV;
	return output;
}

//--------------------------------------------------------------------------
// brief : no lighting vertex shader
// in 	 : 
// out   : 
// note  : nothing
//--------------------------------------------------------------------------
tagPS_INPUT VS_NoLight( tagVS_INPUT input )
{
	tagPS_INPUT output = (tagPS_INPUT)0;
    output.pos = mul( input.pos, wvp );
	output.color = coloration * float4( 1.f , 1.f, 1.f, input.color.a );
	output.diffuseUV = input.diffuseUV;

	return output;
	
}


//- pixel shader -----------------------------------------------------------
//--------------------------------------------------------------------------
// brief : ps with tex
// in 	 : 
// out   : 
// note  : nothing
//--------------------------------------------------------------------------
float4 PS( tagPS_INPUT input ) : COLOR0
{
#if	PER_PIXEL_LIGHTING
	float4 lightParam =
		CalculateLight( input.pos, input.normal, input.color );

	float4	finalColor = tex2D( diffuseTexSampler, input.diffuseUV );
	finalColor *= lightParam * coloration;
#else
	float4	finalColor = coloration * input.color * tex2D( diffuseTexSampler, input.diffuseUV ); 

#endif
	return	finalColor;
}


//--------------------------------------------------------------------------
// brief : ps with no tex
// in 	 : 
// out   : 
// note  : nothing
//--------------------------------------------------------------------------
float4 PS_NoLight( tagPS_INPUT input ) : COLOR0
{
	float4	finalColor = input.color * tex2D( diffuseTexSampler, input.diffuseUV ); 
	return	finalColor;
}


// this is snipet for copy
//--------------------------------------------------------------------------
// brief : 
// in 	 : 
// out   : 
// note  : nothing
//--------------------------------------------------------------------------

//- technique --------------------------------------------------------------
technique Render
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_3_0, VS() ) );
        SetPixelShader( CompileShader( ps_3_0, PS() ) );
    }
}
technique RenderNoLight
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_3_0, VS_NoLight() ) );
        SetPixelShader( CompileShader( ps_3_0, PS_NoLight() ) );
 
   }
}

#if	0
technique Render
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_3_0, VS() ) );
        SetPixelShader( CompileShader( ps_3_0, PS() ) );
    }
}
#endif
